library(testthat)
library(rcest)

sp_correct <- list(ftype = "test_fun",
                   fname = c("fun1", "FUN1"),
                   transforms = c("exp", "none"),
                   def_pars = c(1, 0),
                   par_names = c("p1", "p2"),
                   description = "This is a test function")

test_that("set_parser passes when an correct arguments are passed", {
  expect_invisible(do.call(set_parser, sp_correct))
})

test_that("set_parser fails when an incorrect argument type is passed", {
  clist       <- sp_correct
  clist$ftype <- 1:4
  expect_error(do.call(set_parser, clist), "'ftype' needs to be a character string")

  clist       <- sp_correct
  clist$fname <- 1:4
  expect_error(do.call(set_parser, clist), "'fname' needs to be a character string")

  clist            <- sp_correct
  clist$transforms <- 1:4
  expect_error(do.call(set_parser, clist), "'transforms' needs to be a character string")

  clist          <- sp_correct
  clist$def_pars <- 1:4
  expect_error(do.call(set_parser, clist))

  clist           <- sp_correct
  clist$par_names <- 1:4
  expect_error(do.call(set_parser, clist))

})

test_that("set_parser fails when arguments of list are incorrect length", {
  clist            <- sp_correct
  clist$transforms <- c("none")
  expect_error(do.call(set_parser, clist))

  clist          <- sp_correct
  clist$def_pars <- 1
  expect_error(do.call(set_parser, clist))

  clist          <- sp_correct
  clist$def_pars <- "p1"
  expect_error(do.call(set_parser, clist))
})

test_that("get_parser fails when arguments are of wrong type", {

  expect_error(get_parser(32, "fun1"), "'ftype' arg needs to be a character string")

  expect_error(get_parser("test_fun", 1), "'fname' arg needs to be a character string")

  expect_error(get_parser("one", "fun1"),
               "'one' is not listed in the parser_env. Did you forget to set it first?")

  expect_error(get_parser("test_fun", "one"),
               paste0("fname 'one' is not listed as an available fname for test_fun.",
                      " Did you forget to set it first?"))
})

test_that("get_parser returns a list", {
  expect_is(get_parser(sp_correct$ftype, sp_correct$fname[1]), "list")
  expect_is(get_parser(sp_correct$ftype, sp_correct$fname[2]), "list")
})

test_that("get_parser returns a list with the right names", {
  gp <- get_parser(sp_correct$ftype, sp_correct$fname[1])
  parser_fields <- c("ftype", "fname", "transforms", "def_pars", "par_names", "description")
  expect_setequal(names(get_parser(sp_correct$ftype, sp_correct$fname[1])), names(sp_correct))
  for (n in parser_fields) {
    if (n == "fname") expect_equal(gp[[n]][1], sp_correct[[n]][1])
    else expect_equal(gp[[n]], sp_correct[[n]])
  }
})
