library(testthat)
library(rcest)

###### Parser setting is checked in previous test #####
## Clear the initial environments
set_rcenv()

set_parser(ftype = "test_fun1",
           fname = c("fun11", "FUN11"),
           transforms = c("exp"),
           def_pars = c(1),
           par_names = c("p1"),
           description = "This is a test function with 1 parameter")

set_parser(ftype = "test_fun1",
           fname = c("fun12", "FUN12"),
           transforms = c("exp", "none"),
           def_pars = c(1, 0),
           par_names = c("p1", "p2"),
           description = "This is a test function with 2 parameters")

set_parser(ftype = "test_fun2",
           fname = c("fun21", "FUN21"),
           transforms = c("exp"),
           def_pars = c(1),
           par_names = c("p1"),
           description = "This is a second test function with 1 parameter - exp")

set_parser(ftype = "test_fun2",
           fname = c("fun22", "FUN22"),
           transforms = c("none"),
           def_pars = c(0),
           par_names = c("tp1"),
           description = "This is a second test function with 1 parameter - none")

###### Start by checking ftype_sanitize ####
test_that("ftype_names returns the names of the ftype", {
  ftypes <- list(test_fun1 = "fun11")
  expect_setequal(ftype_names("test_fun1", ftypes), c("p1"))

  ftypes <- list(test_fun1 = "fun12")
  expect_setequal(ftype_names("test_fun1", ftypes), c("p1", "p2"))
})


test_that("ftype_sanitize sanatizes the ftype list", {
  avail_ftypes <- c("test_fun1")
  ftypes <- list(test_fun1 = "fun11")

  sftype <- ftype_sanitize(ftypes)

  expect_is(sftype, "list")
  expect_setequal(names(sftype), avail_ftypes)
})

# Start an ftype list by only testting one parameter
test_that("ftype_index returns a list with the correct names", {
  avail_ftypes <- c("test_fun1", "test_fun2")
  ftypes <- list(test_fun1 = "fun12", test_fun2 = "fun21")

  expect_is(ftype_index(ftypes), "list")
  expect_setequal(names(ftype_index(ftypes)), avail_ftypes)
})

test_that("ftype_index returns the correct number of parameters", {
  ftypes   <- list(test_fun1 = "fun12", test_fun2 = "fun21")
  sftype   <- ftype_sanitize(ftypes)
  ft_index <- ftype_index(sftype)

  expect_length(ft_index$test_fun1, 2)
  expect_length(ft_index$test_fun2, 1)
})

test_that("ftype_index returns the correct index", {
  ftypes   <- list(test_fun1 = "fun12", test_fun2 = "fun21")
  sftype   <- ftype_sanitize(ftypes)
  ft_index <- ftype_index(sftype)

  expect_equal(ft_index$test_fun1, 1:2)
  expect_equal(ft_index$test_fun2, 3)
})
