library(testthat)
library(rcest)

###### Parser setting is checked in previous test #####
# ensure the default environment is set
set_rcdefault()

# Set test functions
set_parser(ftype = "test_fun0",
           fname = c("fun0"),
           transforms  = vector("character", 0),
           def_pars    = vector("numeric",   0),
           par_names   = vector("character", 0),
           description = "This is a test function with 1 parameters")

set_parser(ftype = "test_fun1",
           fname = c("fun12", "FUN12"),
           transforms = c("exp"),
           def_pars = c(1),
           par_names = c("p1"),
           description = "This is a test function with 1 parameters")

set_parser(ftype = "test_fun2",
           fname = c("fun2", "FUN2"),
           transforms = c("exp", "none", "none"),
           def_pars = c(1, 0, 0),
           par_names = c("p1", "p2", "p3"),
           description = "This is a test function with 3 parameters")

# A simulated instrument of binary covariates
inst <- data.frame(dem1 = c(0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L),
                   dem2 = c(0L, 0L, 1L, 0L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 1L),
                   dem3 = c(0L, 0L, 0L, 1L, 1L, 0L, 1L, 1L, 0L, 0L, 1L, 1L, 1L, 1L, 1L))

pmat_res <- matrix(c(2,    5.0,  1,   1,
                     2,    5.0,  1,   1,
                     2.2,  5.5,  1.5, 1.5,
                     2,    5.0,  1,   1,
                     2.31, 5.61, 1.6, 1.6,
                     2,    5.0,  1,   1,
                     2,    5.0,  1,   1,
                     2.2,  5.5,  1.5, 1.5,
                     2,    5.0,  1,   1,
                     2,    5.0,  1,   1,
                     2.1,  5.1,  1.1, 1.1,
                     2.2,  5.5,  1.5, 1.5,
                     2,    5.0,  1,   1,
                     2,    5.0,  1,   1,
                     2.2,  5.5,  1.5, 1.5),
                   ncol = 4, byrow = T)

# Define the ftypes for the simulation
ftypes <- list(test_fun1 = "fun12", test_fun2 = "fun2", test_fun0 = "fun0")
covars <- list(test_fun1 = c("dem1" = .1, "dem2" = .2),
               test_fun2 = c("dem1" = .1, "dem2" = .5))

init <- list(test_fun1 = c(),
             test_fun1 = c(2),
             test_fun2 = c(5, 1, 1))

wrap_opts <- get_wrap_opts(inst       = inst,
                           ftypes     = ftypes,
                           covars     = covars,
                           transforms = list(),
                           init       = init)

pvec <- c(unlist(wrap_opts$made_init), unlist(wrap_opts$made_covars))

test_that("pvec_adjust produces a correctly adjusted pmat when correctly called.", {
  # Adjust for covariates and transforms
  expect_equal(pvec_adjust(pvec, wrap_opts), pmat_res)

})
