library(testthat)
library(rcest)

test_that("Delta Method - variance is correct for easy case", {
  g    <- function(x) exp(x)
  par  <- 0
  sig  <- .25
  dsig <- sqrt(delta_var(g, par, sig^2))
  expect_equal(sig, dsig)
})

test_that("Delta Method - variance is correct for simple multivariate function", {
  g    <- function(par) sum(exp(par))
  par  <- c(0, 0)
  sig  <- matrix(c(1, 0,
                   0, 1), nrow = 2)
  dsig <- delta_var(g, par, sig)
  expect_equal(2, dsig)

  par  <- c(0, 0, 0)
  sig  <- matrix(c(1, 0, 0,
                   0, 1, 0,
                   0, 0, 1), nrow = 3)
  dsig <- delta_var(g, par, sig)
  expect_equal(3, dsig)
})

test_that("Delta Method - covariance is correct for easy case", {
  g1   <- function(x) exp(x[1])
  g2   <- function(x) exp(x[2])
  par  <- c(0, 0)
  sig <- matrix(c(0.25, 0.10,
                  0.10, 0.25), nrow = 2)
  dsig <- delta_cov(g1, g2, par, sig)
  expect_equal(sig, dsig)

  g1   <- function(x) x[1]^2
  g2   <- function(x) (x[2] - 1)^2
  par  <- c(1.5, 1.5)
  sig <- matrix(c(0.25, 0.10,
                  0.10, 0.25), nrow = 2)
  dsig <- delta_cov(g1, g2, par, sig)
  res  <- matrix(c(2.25, 0.30,
                   0.30, 0.25), nrow = 2)
  expect_equal(res, dsig)

})

test_that("Delta Method - covariance is correct for multivariate case", {
  g1   <- function(x) exp(x[1])
  g2   <- function(x) exp(x[2])
  par  <- c(0, 0)
  sig <- matrix(c(0.25, 0.10,
                  0.10, 0.25), nrow = 2)
  dsig <- delta_cov(g1, g2, par, sig)
  expect_equal(sig, dsig)


  g1  <- function(x) x[1]^2
  g2  <- function(x) sum(x^2)
  par <- c(1.5, 1.5)
  sig <- matrix(c(0.25, 0.10,
                  0.10, 0.25), nrow = 2)
  res <- matrix(c(2.25, 3.15,
                  3.15, 6.30), nrow = 2)

  dsig <- delta_cov(g1, g2, par, sig)
  expect_equal(res, dsig)


})
